/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <tiuser.h>
#include <string.h>
#include <signal.h>
#include <memory.h>
#include <unistd.h>

void timeout(int);
extern int t_errno;
extern int initclient(char *);

void
rwho(int argc, char *argv[], char *transport, char *addr,
	uint_t alen)
{
	int fd, i, flags;
	struct t_unitdata *udp;
	struct sigaction sa;

	if (argc < 2) {
		fprintf(stderr, "usage: rwho host [options]\n");
		exit(1);
	}

	/*
	 * Create a client-style transport endpoint.
	 */
	if ((fd = initclient(transport)) < 0)
		exit(1);

	/*
	 * Allocate the t_unitdata structure to be used to
	 * send and receive data.
	 */
	udp = (struct t_unitdata *)t_alloc(fd, T_UNITDATA,
	  T_ALL);
	if (udp == NULL) {
		t_error("Cannot allocate unitdata");
		exit(1);
	}

	/*
	 * Copy the destination address to the unitdata
	 * request.
	 */
	memcpy(udp->addr.buf, addr, alen);
	udp->addr.len = alen;

	/*
	 * Build the command string to send to the server.
	 */
	strcpy(udp->udata.buf, "who");
	for (i = 2; i < argc; i++) {
		strcat(udp->udata.buf, " ");
		strcat(udp->udata.buf, argv[i]);
	}
	udp->udata.buf[strlen(udp->udata.buf)] = '\0';
	udp->udata.len = strlen(udp->udata.buf)+1;

	/*
	 * Set an alarm so we don't block forever.
	 */
	sa.sa_handler = timeout;
	sa.sa_flags = 0;
	sigemptyset(&sa.sa_mask);
	sigaction(SIGALRM, &sa, NULL);
	alarm(20);

	/*
	 * Send the request.
	 */
	if (t_sndudata(fd, udp) < 0) {
		t_error("Cannot send request");
		exit(1);
	}

	/*
	 * Receive response messages until we encounter an
	 * error or receive the last message (a single byte
	 * containing 0).
	 */
	for ( ; ; ) {
		if (t_rcvudata(fd, udp, &flags) < 0) {
			if (t_errno == TLOOK) {
				/*
				 * Try to retrieve the error
				 * indication.
				 */
				if (t_rcvuderr(fd, NULL) < 0) {
					t_error("Cannot clear TLOOK");
					exit(1);
				}
			} else {
				t_error("Cannot receive data");
				exit(1);
			}
		} else {
			/*
			 * If this is the last message, exit.
			 */
			if ((udp->udata.len == 1) &&
			  (*udp->udata.buf == '\0'))
				exit(0);

			/*
			 * Write the response on the standard output.
			 */
			write(1, udp->udata.buf, udp->udata.len);
		}
	}
}

void
timeout(int sig)
{
	fprintf(stderr, "Command timed out\n");
	exit(1);
}
